#include "StdAfx.h"
#include <iostream>
#include "midi.h"
#include "MIDIOutDevice.h"
#include "ShortMsg.h"
#include "chord.h"
#include <string>

using midi::CMIDIOutDevice;
using midi::CShortMsg;

CMIDIOutDevice OutDevice;

bool initialize()
{
	try
	{
		// Make sure there is an output device present
		if(CMIDIOutDevice::GetNumDevs() > 0)
		{

			// Use the first device
			OutDevice.Open(0);
			return(true);
		}
		else
		{
			std::cout << "No MIDI output devices present!\n";
			return(false);
		}
	}
	catch(const std::exception &Err)
	{
		std::cout << Err.what();
		return(false);
	}
}

void play(string notes, int time)
{
	CShortMsg msgs[10];
	for(int i=0;i<notes.size();i++)
	{
		msgs[i]=CShortMsg(midi::NOTE_ON, 0, notes[i], 64, 0);
	}
	for(int i=0;i<notes.size();i++)
	{
		msgs[i].SendMsg(OutDevice);
	}
	::Sleep(time);
	for(int i=0;i<notes.size();i++)
	{
		msgs[i]=CShortMsg(midi::NOTE_OFF, 0, notes[i], 0, 0);
	}
	for(int i=0;i<notes.size();i++)
	{
		msgs[i].SendMsg(OutDevice);
	}
}
void start(string notes){
CShortMsg msgs[10];
	for(int i=0;i<notes.size();i++)
	{
		msgs[i]=CShortMsg(midi::NOTE_ON, 0, notes[i], 64, 0);
	}
	for(int i=0;i<notes.size();i++)
	{
		msgs[i].SendMsg(OutDevice);
	}
}
void end(string notes){
	CShortMsg msgs[10];
	for(int i=0;i<notes.size();i++)
	{
		msgs[i]=CShortMsg(midi::NOTE_OFF, 0, notes[i], 0, 0);
	}
	for(int i=0;i<notes.size();i++)
	{
		msgs[i].SendMsg(OutDevice);
	}
}
HANDLE hStdin; 
DWORD fdwSaveOldMode;
VOID ErrorExit (LPSTR lpszMessage) 
{ 
	fprintf(stderr, "%s\n", lpszMessage); 

	// Restore input mode on exit.

	SetConsoleMode(hStdin, fdwSaveOldMode);

	ExitProcess(0); 
}
chord em4=chord(5,4,1,0,0,0,0,0);
chord em5=chord(6,4,1,0,0,0,0,0);
chord bi4=chord(4,11,0,0,0,0,1,0);
chord bi5=chord(5,11,0,0,0,0,1,0);
chord ci4=chord(5,0,0,0,0,0,1,0);
chord ci5=chord(6,0,0,0,0,0,1,0);
chord eii4=chord(4,7,0,0,0,0,2,0);
chord eii5=chord(5,7,0,0,0,0,2,0);

int root = 0;
int octave = 4;
int _3rd = 0;
int _5th = 0;
int _7th = 2;
int _9th = 1;
int inv = 0;
int extension = 0;
chord pick = chord(octave,root,0,0);
void makeChord(){
	pick = chord(octave, root, _3rd, _5th, _7th, _9th, inv, extension);
}
string trans(int i){
	switch(i){
	case 122:	//z
		return pick.notes.substr(0,1);
		break;
	case 120:	//x
		return pick.notes.substr(1,1);
		break;
	case 99:	//c
		return pick.notes.substr(2,1);
		break;
	case 118:	//v
		return pick.notes.substr(3,1);
		break;
	case 98:	//b
		return pick.notes.substr(4,1);
		break;
	case 110:	//n
		return pick.notes.substr(5,1);
	case 109:	//m
		return pick.notes.substr(6,1);
	case 44:	//,
		return pick.notes.substr(7,1);
	case 46:	//.
	case 47:	// /
	case 0:		// rshift
		break;

	case 43:
		cout << "root up" << endl;
		root++;
		makeChord();
		return "";
	case 45:
		cout << "root down" << endl;
		root--;
		makeChord();
		return "";

	case 97:	//a
		cout << "Major 3rd" << endl;
		_3rd = 0;
		makeChord();
		break;
	case 115:	//s
		cout << "Minor 3rd" << endl;
		_3rd = 1;
		makeChord();
		break;
	case 100:	//d
		cout << "Perfect 5th" << endl;
		_5th = 0;
		makeChord();
		break;
	case 102:	//f
		cout << "Augmented 5th" << endl;
		_5th = 1;
		makeChord();
		break;
	case 103:	//g
		cout << "No 7th" << endl;
		_7th = 0;
		makeChord();
		break;
	case 104:	//h
		cout << "Major 7th" << endl;
		_7th = 2;
		makeChord();
		break;
	case 106:	//j
		cout << "Minor 7th" << endl;
		_7th = 3;
		makeChord();
		break;
	case 107:	//k
		cout << "No 9th" << endl;
		_9th = 0;
		makeChord();
		break;
	case 108:	//l
		cout << "Add9" << endl;
		_9th = 1;
		makeChord();
		break;
	case 59:	//;
		cout << "No inversion" << endl;
		break;

	case 113:	//q
		return em4.notes;
	case 119:	//w
		return bi4.notes;
	case 101:	//e
		return ci4.notes;
	case 114:	//r
		return eii4.notes;
	case 116:	//t
		return em4.notes;
	case 121:	//y
	case 117:	//u
	case 105:	//i
	case 111:	//oqwer
	case 112:	//p
	case 91:	//[
	case 93:	//]
	case 92:	//backslash
		break;
	}
	return "";
}


int main(void)
{
	if(initialize())
	{
		
		string not[12];
		int prog[8]={0,1,0,1,0,2,3,1};

		for(int i=0;i<3;i++)
		{
			not[i].append(1,em4.notes[i]);
			not[i].append(1,em5.notes[2-i]);
			not[i+3].append(1,bi4.notes[i]);
			not[i+3].append(1,bi5.notes[2-i]);
			not[i+6].append(1,ci4.notes[i]);
			not[i+6].append(1,ci5.notes[2-i]);
			not[i+9].append(1,eii4.notes[i]);
			not[i+9].append(1,eii5.notes[2-i]);
		}
		/*play(em4.notes, 400);
		play(bi4.notes, 400);
		play(bi5.notes, 400);
		play(ci4.notes, 400);
		play(ci5.notes, 400);
		play(eii4.notes, 400);
		play(eii5.notes, 400);*/

		DWORD cNumRead, fdwMode, i; 
		INPUT_RECORD irInBuf[1]; 
		int counter=0;

		// Get the standard input handle. 

		hStdin = GetStdHandle(STD_INPUT_HANDLE); 
		if (hStdin == INVALID_HANDLE_VALUE) 
			ErrorExit("GetStdHandle"); 

		// Save the current input mode, to be restored on exit. 

		if (! GetConsoleMode(hStdin, &fdwSaveOldMode) ) 
			ErrorExit("GetConsoleMode"); 

		// Enable the window and mouse input events. 

		fdwMode = ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT; 
		if (! SetConsoleMode(hStdin, fdwMode) ) 
			ErrorExit("SetConsoleMode"); 

		// Loop to read and handle the next 100 input events. 
		while(true){
						// Wait for the events. 

			if (! ReadConsoleInput( 
				hStdin,      // input buffer handle 
				irInBuf,     // buffer to read into 
				1,         // size of read buffer 
				&cNumRead) ) // number of records read 
				ErrorExit("ReadConsoleInput"); 
			// Dispatch the events to the appropriate handler. 

			for (i = 0; i < cNumRead; i++) 
			{
				switch(irInBuf[i].EventType) {
				case KEY_EVENT: // keyboard input
					if(irInBuf[i].Event.KeyEvent.bKeyDown){
						int c = irInBuf[i].Event.KeyEvent.uChar.UnicodeChar;
						cout << c << endl;
						start(trans(c));
						break; 
					}
					else{
						int c = irInBuf[i].Event.KeyEvent.uChar.UnicodeChar;

						end(trans(c));
						break; 
					}
				}

			}
			/*while(true)
			{
			for(int i=0;i<8;i++)
			{
			play(not[0+3*prog[i]],200);
			play(not[1+3*prog[i]],200);
			play(not[2+3*prog[i]],200);
			play(not[1+3*prog[i]],200);
			play(not[0+3*prog[i]],200);
			play(not[1+3*prog[i]],200);
			play(not[2+3*prog[i]],200);
			play(not[1+3*prog[i]],200);
			}
			}*/
		}
		// Close device
		OutDevice.Close();

		return 0;
	}
}